@@ -1,38 +1,43 @@
 use std::{
     any::TypeId,
-    collections::{HashMap, HashSet},
+    collections::{BTreeMap, HashMap, HashSet},
+    rc::Rc,
     sync::Arc,
 };
 
 use parking_lot::Mutex;
 
+mod cache;
 mod computed;
-mod state;
+mod notifiable;
+pub mod tree;
 mod value;
 
-pub use self::state::State;
+pub use self::notifiable::{NotifiableMap, Notify, ScopedNotifiableMap};
 pub use self::value::Value;
 use self::{
+    cache::LayoutCache,
     computed::{ComputedFn, ComputedFunc},
-    state::{StateMap, WidgetStates},
+    tree::Tree,
 };
 
 use crate::{
     layout::Layout,
-    unit::Key,
-    widget::{BuildResult, WidgetId, WidgetRef},
+    plugin::WidgetPlugin,
+    unit::{Shape, Key, LayoutType, Rect},
+    widget::{WidgetId, WidgetRef},
     Ref,
 };
 
 /// A combined-type for anything that can listen for events in the system.
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
-pub enum ListenerID {
+pub enum ListenerId {
     Widget(WidgetId),
     Computed(WidgetId, TypeId),
     Plugin(TypeId),
 }
 
-impl ListenerID {
+impl ListenerId {
     /// # Panics
     ///
     /// Will panic if called on a plugin listener.
@@ -40,7 +45,7 @@ impl ListenerID {
     pub fn widget_id(&self) -> &WidgetId {
         match self {
             Self::Widget(widget_id) | Self::Computed(widget_id, _) => widget_id,
-            ListenerID::Plugin(_) => panic!("listener is not a widget"),
+            ListenerId::Plugin(_) => panic!("listener is not a widget"),
         }
     }
 }
@@ -49,24 +54,39 @@ type WidgetComputedFuncs<'ui> =
     HashMap<WidgetId, HashMap<TypeId, Box<dyn ComputedFunc<'ui> + 'ui>>>;
 
 pub struct WidgetContext<'ui> {
-    global: StateMap,
-    states: WidgetStates,
+    pub(crate) tree: Tree<WidgetId>,
+    pub(crate) cache: LayoutCache<WidgetId>,
+
+    pub(crate) plugins: Mutex<BTreeMap<TypeId, Rc<dyn WidgetPlugin>>>,
+
+    global: NotifiableMap,
+    states: ScopedNotifiableMap<WidgetId>,
 
     layouts: Mutex<HashMap<WidgetId, Ref<Layout>>>,
+    layout_types: Mutex<HashMap<WidgetId, Ref<LayoutType>>>,
+    clipping: Mutex<HashMap<WidgetId, Ref<Shape>>>,
 
     computed_funcs: Arc<Mutex<WidgetComputedFuncs<'ui>>>,
 
-    pub(crate) current_id: Arc<Mutex<Option<ListenerID>>>,
+    pub(crate) current_id: Arc<Mutex<Option<ListenerId>>>,
 }
 
 impl<'ui> WidgetContext<'ui> {
     #[must_use]
     #[allow(clippy::needless_pass_by_value)]
-    pub fn new(changed: Arc<Mutex<HashSet<ListenerID>>>) -> Self {
+    pub fn new(changed: Arc<Mutex<HashSet<ListenerId>>>) -> Self {
         Self {
-            global: StateMap::new(Arc::clone(&changed)),
-            states: WidgetStates::new(Arc::clone(&changed)),
+            tree: Tree::default(),
+            cache: LayoutCache::default(),
+
+            plugins: Mutex::default(),
+
+            global: NotifiableMap::new(Arc::clone(&changed)),
+            states: ScopedNotifiableMap::new(Arc::clone(&changed)),
+
             layouts: Mutex::default(),
+            layout_types: Mutex::default(),
+            clipping: Mutex::default(),
 
             computed_funcs: Arc::new(Mutex::new(HashMap::new())),
 
@@ -74,39 +94,136 @@ impl<'ui> WidgetContext<'ui> {
         }
     }
 
-    // Global and local state
+    /// Returns the widget tree.
+    pub fn get_tree(&self) -> &Tree<WidgetId> {
+        &self.tree
+    }
 
-    /// Initializing a global does not cause the initializer to be updated when its value is changed.
-    pub fn init_global<V>(&self) -> State<V>
+    /// Get the visual `Rect` of a widget.
+    pub fn get_rect(&self, widget_id: &WidgetId) -> Option<&Rect> {
+        self.cache.get_rect(widget_id)
+    }
+
+    /// # Panics
+    ///
+    /// Will panic if called outside of a widget build context, or in a plugin.
+    pub fn get_self(&self) -> WidgetId {
+        let current_id = self
+            .current_id
+            .lock()
+            .expect("cannot get self while not iterating");
+
+        match current_id {
+            ListenerId::Widget(widget_id) | ListenerId::Computed(widget_id, _) => widget_id,
+            ListenerId::Plugin(_) => {
+                panic!("plugins do not exist in the tree, and thus they cannot get themselves")
+            }
+        }
+    }
+
+    /// # Panics
+    ///
+    /// Will panic if called outside of a widget build context, or in a plugin.
+    pub fn is_self(&self, widget_id: WidgetId) -> bool {
+        self.get_self() == widget_id
+    }
+
+    pub(crate) fn remove(&mut self, widget_id: &WidgetId) {
+        let listener_id = ListenerId::Widget(*widget_id);
+
+        self.cache.remove(widget_id);
+
+        self.global.remove_listener(&listener_id);
+
+        self.states.remove(widget_id);
+
+        self.states.remove_listeners(&listener_id);
+
+        self.layouts.lock().remove(widget_id);
+
+        self.computed_funcs.lock().remove(widget_id);
+    }
+}
+
+// Plugins
+impl<'ui> WidgetContext<'ui> {
+    /// Initialize a plugin if it's not set already.
+    pub fn init_plugin<P, F>(&self, func: F) -> Rc<P>
     where
-        V: Value + Default,
+        P: WidgetPlugin,
+        F: FnOnce() -> P,
+    {
+        if self.plugins.lock().contains_key(&TypeId::of::<P>()) {
+            self.get_plugin::<P>()
+                .expect("failed to get initialized plugin")
+        } else {
+            let plugin_id = TypeId::of::<P>();
+
+            let plugin = func();
+
+            let last_id = *self.current_id.lock();
+
+            *self.current_id.lock() = Some(ListenerId::Plugin(plugin_id));
+
+            plugin.on_update(self);
+
+            *self.current_id.lock() = last_id;
+
+            self.plugins.lock().insert(plugin_id, Rc::new(plugin));
+
+            self.get_plugin::<P>().expect("failed to set plugin")
+        }
+    }
+
+    /// Fetch a plugin, or initialize it with `func`.
+    pub fn get_plugin_or<P, F>(&self, func: F) -> Rc<P>
+    where
+        P: WidgetPlugin,
+        F: FnOnce() -> P,
+    {
+        self.get_plugin::<P>()
+            .map_or_else(|| self.init_plugin(func), |plugin| plugin)
+    }
+
+    /// Fetch a plugin if it exists.
+    pub fn get_plugin<P>(&self) -> Option<Rc<P>>
+    where
+        P: WidgetPlugin,
+    {
+        match Rc::clone(self.plugins.lock().get(&TypeId::of::<P>())?).downcast_rc::<P>() {
+            Ok(plugin) => Some(plugin),
+            Err(..) => None,
+        }
+    }
+}
+
+// Global state
+impl<'ui> WidgetContext<'ui> {
+    /// Initialize a global value if it's not set already. This does not cause the initializer to be updated when its value is changed.
+    pub fn init_global<V, F>(&self, func: F) -> Notify<V>
+    where
+        V: Value,
+        F: FnOnce() -> V,
     {
         if !self.global.contains::<V>() {
-            self.global.insert(V::default());
+            self.global.set(func());
         }
 
         self.global.get::<V>().expect("failed to init global")
     }
 
-    pub fn set_global<V>(&self, value: V) -> State<V>
+    /// Fetch a global value, or initialize it with `func`. The caller will be updated when the value is changed.
+    pub fn use_global<V, F>(&self, func: F) -> Notify<V>
     where
         V: Value,
+        F: FnOnce() -> V,
     {
-        self.global.insert(value);
-
-        self.get_global()
-            .expect("failed to fetch global, this shouldn't be possible")
+        self.try_use_global::<V>()
+            .map_or_else(|| self.init_global(func), |v| v)
     }
 
-    pub fn get_or_init_global<V>(&self) -> State<V>
-    where
-        V: Value + Default,
-    {
-        self.get_global::<V>()
-            .map_or_else(|| self.set_global(V::default()), |v| v)
-    }
-
-    pub fn get_global<V>(&self) -> Option<State<V>>
+    /// Fetch a global value if it exists. The caller will be updated when the value is changed.
+    pub fn try_use_global<V>(&self) -> Option<Notify<V>>
     where
         V: Value,
     {
@@ -116,27 +233,12 @@ impl<'ui> WidgetContext<'ui> {
 
         self.global.get::<V>()
     }
+}
 
-    pub fn set_state<V>(&self, value: V) -> State<V>
-    where
-        V: Value,
-    {
-        let current_id = self
-            .current_id
-            .lock()
-            .expect("cannot get state from context while not iterating");
-
-        self.states.set(&current_id, value)
-    }
-
-    pub fn get_state<V: Default>(&self) -> State<V>
-    where
-        V: Value,
-    {
-        self.get_state_or(V::default)
-    }
-
-    pub fn get_state_or<V, F>(&self, func: F) -> State<V>
+// Local state
+impl<'ui> WidgetContext<'ui> {
+    /// Initializing a state does not cause the initializer to be updated when its value is changed.
+    pub fn init_state<V, F>(&self, func: F) -> Notify<V>
     where
         V: Value,
         F: FnOnce() -> V,
@@ -146,11 +248,80 @@ impl<'ui> WidgetContext<'ui> {
             .lock()
             .expect("cannot get state from context while not iterating");
 
-        self.states.get(&current_id, func)
+        let widget_id = *current_id.widget_id();
+
+        self.states.get(widget_id, func)
     }
 
-    // Layout
+    /// Fetch a local state value, or initialize it with `func` if it doesn't exist. The caller will be updated when the value is changed.
+    pub fn use_state<V, F>(&self, func: F) -> Notify<V>
+    where
+        V: Value,
+        F: FnOnce() -> V,
+    {
+        let current_id = self
+            .current_id
+            .lock()
+            .expect("cannot get state from context while not iterating");
 
+        let widget_id = *current_id.widget_id();
+
+        self.states.add_listener::<V>(widget_id, current_id);
+
+        self.states.get(widget_id, func)
+    }
+
+    pub fn get_state_for<V, F>(&self, widget_id: WidgetId, func: F) -> Notify<V>
+    where
+        V: Value,
+        F: FnOnce() -> V,
+    {
+        let current_id = self
+            .current_id
+            .lock()
+            .expect("cannot get state from context while not iterating");
+
+        self.states.add_listener::<V>(widget_id, current_id);
+
+        self.states.get(widget_id, func)
+    }
+}
+
+// Layout
+impl<'ui> WidgetContext<'ui> {
+    /// Set the layout type of the widget.
+    ///
+    /// Used in a `build()` method to set the layout type of the widget being built.
+    pub fn set_layout_type(&self, layout_type: Ref<LayoutType>) {
+        let current_id = self
+            .current_id
+            .lock()
+            .expect("cannot get state from context while not iterating");
+
+        match &current_id {
+            ListenerId::Widget(widget_id) => {
+                self.layout_types.lock().insert(*widget_id, layout_type);
+            }
+            ListenerId::Computed(_, _) => {
+                log::warn!("layout types set in a computed function are ignored");
+            }
+            ListenerId::Plugin(_) => {
+                log::warn!("layout types set in a plugin are ignored");
+            }
+        };
+    }
+
+    /// Fetch the layout of a widget.
+    pub fn get_layout_type(&self, widget_id: &WidgetId) -> Ref<LayoutType> {
+        self.layout_types
+            .lock()
+            .get(widget_id)
+            .map_or(Ref::None, Ref::clone)
+    }
+
+    /// Set the layout of the widget.
+    ///
+    /// Used in a `build()` method to set the layout of the widget being built.
     pub fn set_layout(&self, layout: Ref<Layout>) {
         let current_id = self
             .current_id
@@ -158,33 +329,68 @@ impl<'ui> WidgetContext<'ui> {
             .expect("cannot get state from context while not iterating");
 
         match &current_id {
-            ListenerID::Widget(widget_id) => {
+            ListenerId::Widget(widget_id) => {
                 self.layouts.lock().insert(*widget_id, layout);
             }
-            ListenerID::Computed(_, _) => {
+            ListenerId::Computed(_, _) => {
                 log::warn!("layouts set in a computed function are ignored");
             }
-            ListenerID::Plugin(_) => {
+            ListenerId::Plugin(_) => {
                 log::warn!("layouts set in a plugin are ignored");
             }
         };
     }
 
+    /// Fetch the layout of a widget.
     pub fn get_layout(&self, widget_id: &WidgetId) -> Ref<Layout> {
         self.layouts
             .lock()
             .get(widget_id)
             .map_or(Ref::None, Ref::clone)
     }
+}
 
-    // Computed
+// Clipping
+impl<'ui> WidgetContext<'ui> {
+    /// Set the clipping mask of the widget.
+    ///
+    /// Used in a `build()` method to set the clipping mask of the widget being built.
+    pub fn set_clipping(&self, clipping: Ref<Shape>) {
+        let current_id = self
+            .current_id
+            .lock()
+            .expect("cannot get state from context while not iterating");
 
+        match &current_id {
+            ListenerId::Widget(widget_id) => {
+                self.clipping.lock().insert(*widget_id, clipping);
+            }
+            ListenerId::Computed(_, _) => {
+                log::warn!("layouts set in a computed function are ignored");
+            }
+            ListenerId::Plugin(_) => {
+                log::warn!("layouts set in a plugin are ignored");
+            }
+        };
+    }
+
+    /// Fetch the clipping mask of a widget.
+    pub fn get_clipping(&self, widget_id: &WidgetId) -> Ref<Shape> {
+        self.clipping
+            .lock()
+            .get(widget_id)
+            .map_or(Ref::None, Ref::clone)
+    }
+}
+
+// Computed
+impl<'ui> WidgetContext<'ui> {
     /// # Panics
     ///
     /// Will panic if called outside of a build context.
     pub fn computed<V, F>(&self, func: F) -> V
     where
-        V: Eq + PartialEq + Copy + Clone + Value,
+        V: Eq + PartialEq + Copy + Value,
         F: Fn(&Self) -> V + 'ui + 'static,
     {
         let current_id = self
@@ -196,7 +402,7 @@ impl<'ui> WidgetContext<'ui> {
 
         let computed_id = TypeId::of::<F>();
 
-        let listener_id = ListenerID::Computed(*widget_id, computed_id);
+        let listener_id = ListenerId::Computed(*widget_id, computed_id);
 
         let mut widgets = self.computed_funcs.lock();
 
@@ -226,9 +432,10 @@ impl<'ui> WidgetContext<'ui> {
             .and_then(|widgets| widgets.get_mut(&computed_id))
             .map_or(false, |computed_func| computed_func.call(self))
     }
+}
 
-    // Keys
-
+// Keys
+impl<'ui> WidgetContext<'ui> {
     /// # Panics
     ///
     /// Will panic if called outside of a widget build context.
@@ -249,54 +456,4 @@ impl<'ui> WidgetContext<'ui> {
             widget: Box::new(widget),
         }
     }
-
-    // Other
-
-    /// # Panics
-    ///
-    /// Will panic if called outside of a widget build context, or in a plugin.
-    pub fn get_self(&self) -> WidgetId {
-        let current_id = self
-            .current_id
-            .lock()
-            .expect("cannot get self while not iterating");
-
-        match current_id {
-            ListenerID::Widget(widget_id) | ListenerID::Computed(widget_id, _) => widget_id,
-            ListenerID::Plugin(_) => {
-                panic!("plugins do not exist in the tree, and thus they cannot get themselves")
-            }
-        }
-    }
-
-    /// # Panics
-    ///
-    /// Will panic if called outside of a widget build context, or in a plugin.
-    pub fn is_self(&self, widget_id: WidgetId) -> bool {
-        self.get_self() == widget_id
-    }
-
-    pub(crate) fn build(&mut self, widget_id: WidgetId, widget: &WidgetRef) -> BuildResult {
-        *self.current_id.lock() = Some(ListenerID::Widget(widget_id));
-
-        let result = widget
-            .try_get()
-            .map_or(BuildResult::Empty, |widget| widget.build(self));
-
-        *self.current_id.lock() = None;
-
-        result
-    }
-
-    pub(crate) fn remove(&mut self, widget_id: &WidgetId) {
-        let listener_id = ListenerID::Widget(*widget_id);
-
-        self.global.remove_listener(&listener_id);
-
-        self.states.remove(widget_id);
-
-        self.layouts.lock().remove(widget_id);
-
-        self.computed_funcs.lock().remove(widget_id);
-    }
 }
