@@ -1,6 +1,6 @@
 use std::any::TypeId;
 
-use crate::{widget::WidgetId, unit::Rect};
+use crate::widget::WidgetId;
 
 /// Used to indicate a change to widgets in the tree.
 #[derive(Copy, Clone, Debug)]
@@ -11,17 +11,17 @@ pub enum WidgetEvent {
         type_id: TypeId,
         widget_id: WidgetId,
     },
-    
+
     /// A widget has changed in the layout.
     Layout {
         type_id: TypeId,
         widget_id: WidgetId,
-        rect: Rect,
+        layer: u32,
     },
 
     /// A widget has been destroyed.
     Destroyed {
         type_id: TypeId,
         widget_id: WidgetId,
-    }
+    },
 }
