@@ -1,7 +1,4 @@
-use crate::{
-    unit::{Padding, Position, Sizing},
-    Ref,
-};
+use crate::unit::{Margin, Position, Sizing};
 
 /// Holds layout parameters to dictate how the element should be displayed.
 #[derive(Debug, Clone, Default)]
@@ -11,11 +8,5 @@ pub struct Layout {
     pub max_sizing: Sizing,
     pub sizing: Sizing,
 
-    pub padding: Padding,
-}
-
-impl From<Layout> for Ref<Layout> {
-    fn from(layout: Layout) -> Self {
-        Self::new(layout)
-    }
+    pub margin: Margin,
 }
