@@ -1,49 +1,33 @@
 use std::{
-    any::TypeId,
-    collections::{BTreeMap, HashMap, HashSet},
+    collections::{HashMap, HashSet},
     rc::Rc,
     sync::Arc,
 };
 
 use generational_arena::Arena;
-use morphorm::{Cache, Units};
+use morphorm::Cache;
 use parking_lot::Mutex;
 
 use crate::{
-    context::{ListenerID, WidgetContext},
+    context::{tree::Tree, ListenerId, WidgetContext},
     event::WidgetEvent,
-    plugin::WidgetPlugin,
-    unit::{Key, Rect},
-    widget::{Widget, WidgetId, WidgetRef},
+    unit::{Key, Rect, Shape, Units},
+    widget::{BuildResult, Widget, WidgetId, WidgetRef},
     Ref,
 };
 
-mod cache;
 mod debug;
 mod node;
-mod tree;
 
 pub use self::node::WidgetNode;
 
-use self::{cache::LayoutCache, tree::Tree};
-
-struct VoidEvents;
-
-impl Extend<WidgetEvent> for VoidEvents {
-    fn extend<T: IntoIterator<Item = WidgetEvent>>(&mut self, _: T) {}
-}
-
 /// Handles the entirety of the widget lifecycle.
 pub struct WidgetManager<'ui> {
-    plugins: BTreeMap<TypeId, Box<dyn WidgetPlugin>>,
-
     widgets: Arena<WidgetNode>,
-    tree: Tree<WidgetId>,
-    cache: LayoutCache<WidgetId>,
 
     context: WidgetContext<'ui>,
 
-    changed: Arc<Mutex<HashSet<ListenerID>>>,
+    changed: Arc<Mutex<HashSet<ListenerId>>>,
 
     modifications: Vec<Modify>,
 
@@ -65,11 +49,7 @@ impl<'ui> Default for WidgetManager<'ui> {
         let changed = Arc::new(Mutex::new(HashSet::new()));
 
         Self {
-            plugins: BTreeMap::default(),
-
             widgets: Arena::default(),
-            tree: Tree::default(),
-            cache: LayoutCache::default(),
 
             context: WidgetContext::new(Arc::clone(&changed)),
 
@@ -99,40 +79,9 @@ impl<'ui> WidgetManager<'ui> {
         Self::default()
     }
 
-    /// Initialize a plugin with its `Default::default()` method.
-    pub fn init_plugin<P>(&mut self)
-    where
-        P: WidgetPlugin + Default,
-    {
-        self.add_plugin(P::default());
-    }
-
-    /// Load a plugin.
-    pub fn add_plugin<P>(&mut self, plugin: P)
-    where
-        P: WidgetPlugin,
-    {
-        let plugin_id = TypeId::of::<P>();
-        let plugin = Box::new(plugin);
-
-        self.plugins.insert(plugin_id, plugin);
-
-        self.update_plugin(plugin_id);
-    }
-
-    /// Fetch a loaded plugin. Will be `None` if it does not exist.
-    pub fn get_plugin<P>(&self) -> Option<&P>
-    where
-        P: WidgetPlugin,
-    {
-        self.plugins
-            .get(&TypeId::of::<P>())
-            .and_then(|plugin| plugin.downcast_ref())
-    }
-
     /// Returns the widget tree.
     pub fn get_tree(&self) -> &Tree<WidgetId> {
-        &self.tree
+        self.context.get_tree()
     }
 
     /// Check if a widget exists in the tree.
@@ -146,9 +95,9 @@ impl<'ui> WidgetManager<'ui> {
     }
 
     /// Fetch the tree representation of a widget.
-    /// 
+    ///
     /// # Panics
-    /// 
+    ///
     /// Will panic if the widget is not found.
     pub fn get_node(&self, widget_id: &WidgetId) -> &WidgetNode {
         self.widgets
@@ -164,9 +113,9 @@ impl<'ui> WidgetManager<'ui> {
     }
 
     /// Fetch a widget from the tree.
-    /// 
+    ///
     /// # Panics
-    /// 
+    ///
     /// This will panic if the widget is not found.
     pub fn get(&self, widget_id: &WidgetId) -> WidgetRef {
         self.try_get(widget_id).expect("widget does not exist")
@@ -182,9 +131,9 @@ impl<'ui> WidgetManager<'ui> {
     }
 
     /// Fetch a widget as the specified type.
-    /// 
+    ///
     /// # Panics
-    /// 
+    ///
     /// If the widget is not the requested type, it will panic.
     pub fn get_as<W>(&self, widget_id: &WidgetId) -> Rc<W>
     where
@@ -195,7 +144,12 @@ impl<'ui> WidgetManager<'ui> {
 
     /// Get the visual `Rect` of a widget.
     pub fn get_rect(&self, widget_id: &WidgetId) -> Option<&Rect> {
-        self.cache.get_rect(widget_id)
+        self.context.get_rect(widget_id)
+    }
+
+    /// Get the visual clipping `Path` for a widget.
+    pub fn get_clipping(&self, widget_id: &WidgetId) -> Ref<Shape> {
+        self.context.get_clipping(widget_id)
     }
 
     /// Get the widget build context.
@@ -211,7 +165,7 @@ impl<'ui> WidgetManager<'ui> {
 
         if parent_id.is_none() {
             // Check if we already have a root node, and queue it for removal if so
-            if let Some(root_id) = self.tree.get_root() {
+            if let Some(root_id) = self.get_tree().get_root() {
                 self.modifications.push(Modify::Destroy(root_id));
             }
         }
@@ -225,181 +179,218 @@ impl<'ui> WidgetManager<'ui> {
     }
 
     /// Update the UI tree.
-    /// 
+    ///
     /// This processes any pending additions, removals, and updates. The `events` parameter is a list of all
     /// changes that occured during the process, in order.
-    pub fn update<E>(&mut self, events: &mut E)
-    where
-        E: Extend<WidgetEvent>,
-    {
+    #[allow(clippy::too_many_lines)]
+    pub fn update(&mut self, events: &mut Vec<WidgetEvent>) {
         if self.modifications.is_empty() && self.changed.lock().is_empty() {
             return;
         }
 
+        let mut root_changed = false;
+
         // Update everything until all widgets fall into a stable state. Incorrectly set up widgets may
         // cause an infinite loop, so be careful.
-        loop {
-            // Apply any queued modifications
-            self.apply_modifications(events);
+        //
+        // We have two loops here because plugins may cause additional modifications after layout
+        'layout: loop {
+            'modify: loop {
+                // Apply any queued modifications
+                self.apply_modifications(events);
 
-            let changed = self.changed.lock().drain().collect::<Vec<_>>();
+                let changed = self.changed.lock().drain().collect::<Vec<_>>();
 
-            if changed.is_empty() {
-                break;
-            }
-
-            cfg_if::cfg_if! {
-                if #[cfg(test)] {
-                    self.changes += changed.len();
+                if changed.is_empty() {
+                    break 'modify;
                 }
-            }
 
-            let mut dirty_widgets = HashSet::new();
-
-            for listener_id in changed {
-                match listener_id {
-                    ListenerID::Widget(widget_id) => {
-                        dirty_widgets.insert(widget_id);
+                cfg_if::cfg_if! {
+                    if #[cfg(test)] {
+                        self.changes += changed.len();
                     }
+                }
 
-                    ListenerID::Computed(widget_id, computed_id) => {
-                        if self.context.call_computed_func(&widget_id, computed_id) {
+                let mut dirty_widgets = HashSet::new();
+
+                for listener_id in changed {
+                    match listener_id {
+                        ListenerId::Widget(widget_id) => {
                             dirty_widgets.insert(widget_id);
                         }
-                    }
 
-                    ListenerID::Plugin(plugin_id) => {
-                        self.update_plugin(plugin_id);
-                    }
-                }
-            }
-
-            let mut to_rebuild = Vec::new();
-
-            'main: for widget_id in dirty_widgets {
-                let node = match self.tree.get(&widget_id) {
-                    Some(widget) => widget,
-                    None => continue,
-                };
-
-                let widget_depth = node.depth;
-
-                let mut to_remove = Vec::new();
-
-                for (i, &(dirty_id, dirty_depth)) in to_rebuild.iter().enumerate() {
-                    // If they're at the same depth, bail. No reason to check if they're children.
-                    if widget_depth == dirty_depth {
-                        continue;
-                    }
-
-                    if widget_depth > dirty_depth {
-                        // If the widget is a child of one of the already queued widgets, bail. It's
-                        // already going to be updated.
-                        if self.tree.has_child(&dirty_id, &widget_id) {
-                            continue 'main;
+                        ListenerId::Computed(widget_id, computed_id) => {
+                            if self.context.call_computed_func(&widget_id, computed_id) {
+                                dirty_widgets.insert(widget_id);
+                            }
                         }
-                    } else {
-                        // If the widget is a parent of the widget already queued for render, remove it
-                        if self.tree.has_child(&widget_id, &dirty_id) {
-                            to_remove.push(i);
+
+                        ListenerId::Plugin(plugin_id) => {
+                            let plugins = self.context.plugins.lock();
+
+                            let plugin = plugins
+                                .get(&plugin_id)
+                                .expect("cannot update a plugin that does not exist");
+
+                            *self.context.current_id.lock() = Some(ListenerId::Plugin(plugin_id));
+
+                            plugin.on_update(&self.context);
+
+                            *self.context.current_id.lock() = None;
                         }
                     }
                 }
 
-                // Remove the queued widgets that will be updated as a consequence of updating `widget`
-                for (offset, index) in to_remove.into_iter().enumerate() {
-                    to_rebuild.remove(index - offset);
+                let mut to_rebuild = Vec::new();
+
+                'main: for widget_id in dirty_widgets {
+                    let node = match self.get_tree().get(&widget_id) {
+                        Some(widget) => widget,
+                        None => continue,
+                    };
+
+                    let widget_depth = node.depth;
+
+                    let mut to_remove = Vec::new();
+
+                    for (i, &(dirty_id, dirty_depth)) in to_rebuild.iter().enumerate() {
+                        // If they're at the same depth, bail. No reason to check if they're children.
+                        if widget_depth == dirty_depth {
+                            continue;
+                        }
+
+                        if widget_depth > dirty_depth {
+                            // If the widget is a child of one of the already queued widgets, bail. It's
+                            // already going to be updated.
+                            if self.get_tree().has_child(&dirty_id, &widget_id) {
+                                continue 'main;
+                            }
+                        } else {
+                            // If the widget is a parent of the widget already queued for render, remove it
+                            if self.get_tree().has_child(&widget_id, &dirty_id) {
+                                to_remove.push(i);
+                            }
+                        }
+                    }
+
+                    // Remove the queued widgets that will be updated as a consequence of updating `widget`
+                    for (offset, index) in to_remove.into_iter().enumerate() {
+                        to_rebuild.remove(index - offset);
+                    }
+
+                    to_rebuild.push((widget_id, widget_depth));
                 }
 
-                to_rebuild.push((widget_id, widget_depth));
+                for (widget_id, _) in to_rebuild {
+                    self.modifications.push(Modify::Rebuild(widget_id));
+                }
             }
 
-            for (widget_id, _) in to_rebuild {
-                self.modifications.push(Modify::Rebuild(widget_id));
+            // Workaround for morphorm ignoring root sizing
+            if self.morphorm_root_workaround() {
+                root_changed = true;
+            }
+
+            morphorm::layout(&mut self.context.cache, &self.context.tree, &self.widgets);
+
+            let plugins = self.context.plugins.lock();
+
+            for (plugin_id, plugin) in plugins.iter() {
+                *self.context.current_id.lock() = Some(ListenerId::Plugin(*plugin_id));
+
+                plugin.on_layout(&self.context);
+            }
+
+            *self.context.current_id.lock() = None;
+
+            if self.modifications.is_empty() {
+                break 'layout;
             }
         }
 
-        // Workaround for morphorm ignoring root sizing
-        let root_changed = self.morphorm_root_workaround();
-
-        morphorm::layout(&mut self.cache, &self.tree, &self.widgets);
-
-        events.extend(self.cache.take_changed().iter().map(|widget_id| {
-            WidgetEvent::Layout {
-                type_id: self.get(widget_id).get_type_id(),
-                widget_id: *widget_id,
-                rect: *self
-                    .cache
-                    .get_rect(widget_id)
-                    .expect("root widget does not have a rect"),
-            }
-        }));
+        let mut changed = self.context.cache.take_changed();
 
         if root_changed {
-            if let Some(widget_id) = self.tree.get_root() {
-                events.extend(Some(WidgetEvent::Layout {
-                    type_id: self.get(&widget_id).get_type_id(),
-                    widget_id,
-                    rect: *self
-                        .cache
-                        .get_rect(&widget_id)
-                        .expect("root widget does not have a rect"),
-                }));
+            if let Some(widget_id) = self.get_tree().get_root() {
+                changed.insert(widget_id);
             }
         }
+
+        events.extend(
+            changed
+                .into_iter()
+                .filter(|widget_id| self.contains(widget_id))
+                .map(|widget_id| {
+                    let type_id = self.get(&widget_id).get_type_id();
+                    let layer = self.get_node(&widget_id).layer;
+
+                    WidgetEvent::Layout {
+                        type_id,
+                        widget_id,
+                        layer,
+                    }
+                }),
+        );
+
+        let plugins = self.context.plugins.lock();
+
+        for (plugin_id, plugin) in plugins.iter() {
+            *self.context.current_id.lock() = Some(ListenerId::Plugin(*plugin_id));
+
+            plugin.on_events(&self.context, events);
+        }
+
+        *self.context.current_id.lock() = None;
     }
 
     fn morphorm_root_workaround(&mut self) -> bool {
         let mut root_changed = false;
 
-        if let Some(widget_id) = self.tree.get_root() {
+        if let Some(widget_id) = self.get_tree().get_root() {
             if let Some(layout) = self.context.get_layout(&widget_id).try_get() {
                 if let Units::Pixels(px) = layout.position.get_left() {
-                    if (self.cache.posx(widget_id) - px).abs() > f32::EPSILON {
+                    if (self.context.cache.posx(widget_id) - px).abs() > f32::EPSILON {
                         root_changed = true;
 
-                        self.cache.set_posx(widget_id, px);
+                        self.context.cache.set_posx(widget_id, px);
                     }
                 }
 
                 if let Units::Pixels(px) = layout.position.get_top() {
-                    if (self.cache.posy(widget_id) - px).abs() > f32::EPSILON {
+                    if (self.context.cache.posy(widget_id) - px).abs() > f32::EPSILON {
                         root_changed = true;
 
-                        self.cache.set_posy(widget_id, px);
+                        self.context.cache.set_posy(widget_id, px);
                     }
                 }
 
                 if let Units::Pixels(px) = layout.sizing.get_width() {
-                    if (self.cache.width(widget_id) - px).abs() > f32::EPSILON {
+                    if (self.context.cache.width(widget_id) - px).abs() > f32::EPSILON {
                         root_changed = true;
 
-                        self.cache.set_width(widget_id, px);
+                        self.context.cache.set_width(widget_id, px);
                     }
                 }
 
                 if let Units::Pixels(px) = layout.sizing.get_height() {
-                    if (self.cache.height(widget_id) - px).abs() > f32::EPSILON {
+                    if (self.context.cache.height(widget_id) - px).abs() > f32::EPSILON {
                         root_changed = true;
 
-                        self.cache.set_height(widget_id, px);
+                        self.context.cache.set_height(widget_id, px);
                     }
                 }
             } else {
-                self.cache.set_posx(widget_id, 0.0);
-                self.cache.set_posy(widget_id, 0.0);
-                self.cache.set_width(widget_id, 0.0);
-                self.cache.set_height(widget_id, 0.0);
+                self.context.cache.set_posx(widget_id, 0.0);
+                self.context.cache.set_posy(widget_id, 0.0);
+                self.context.cache.set_width(widget_id, 0.0);
+                self.context.cache.set_height(widget_id, 0.0);
             }
         }
 
         root_changed
     }
 
-    fn apply_modifications<E>(&mut self, events: &mut E)
-    where
-        E: Extend<WidgetEvent>,
-    {
+    fn apply_modifications(&mut self, events: &mut Vec<WidgetEvent>) {
         let mut removed_keyed = HashMap::new();
 
         while !self.modifications.is_empty() {
@@ -452,28 +443,13 @@ impl<'ui> WidgetManager<'ui> {
         }
     }
 
-    fn update_plugin(&mut self, plugin_id: TypeId) {
-        let plugin = self
-            .plugins
-            .get(&plugin_id)
-            .expect("cannot update a plugin that does not exist");
-
-        *self.context.current_id.lock() = Some(ListenerID::Plugin(plugin_id));
-
-        plugin.on_update(self, &self.context);
-
-        *self.context.current_id.lock() = None;
-    }
-
-    fn process_spawn<E>(
+    fn process_spawn(
         &mut self,
-        events: &mut E,
+        events: &mut Vec<WidgetEvent>,
         removed_keyed: &mut HashMap<(Option<WidgetId>, Key), WidgetId>,
         parent_id: Option<WidgetId>,
         widget: WidgetRef,
-    ) where
-        E: Extend<WidgetEvent>,
-    {
+    ) {
         if parent_id.is_some() && !self.contains(parent_id.as_ref().unwrap()) {
             panic!("cannot add a widget to a nonexistent parent")
         }
@@ -482,7 +458,7 @@ impl<'ui> WidgetManager<'ui> {
         if let WidgetRef::Keyed { owner_id, key, .. } = &widget {
             if let Some(keyed_id) = removed_keyed.remove(&(*owner_id, *key)) {
                 // Reparent the (removed) keyed widget to the new widget
-                self.tree.reparent(parent_id, keyed_id);
+                self.context.tree.reparent(parent_id, keyed_id);
 
                 return;
             }
@@ -490,34 +466,46 @@ impl<'ui> WidgetManager<'ui> {
 
         let type_id = widget.get_type_id();
 
-        let layout_type = widget.get().layout_type();
+        let widget_id = WidgetId::from(self.widgets.insert(WidgetNode {
+            widget,
+            layer: 0,
+            layout_type: Ref::None,
+            layout: Ref::None,
+        }));
 
-        let widget_id = WidgetId::from(
-            self.widgets.insert(WidgetNode {
-                widget,
-                layout_type,
-                layout: Ref::None,
-            }),
-            parent_id.map_or(0, |node| node.depth() + 1),
-        );
-
-        self.tree.add(parent_id, widget_id);
+        self.context.tree.add(parent_id, widget_id);
 
         // Sometimes widgets get changes queued before they're spawned
-        self.changed.lock().remove(&ListenerID::Widget(widget_id));
+        self.changed.lock().remove(&ListenerId::Widget(widget_id));
 
         self.modifications.push(Modify::Rebuild(widget_id));
 
-        events.extend(Some(WidgetEvent::Spawned { type_id, widget_id }));
+        events.push(WidgetEvent::Spawned { type_id, widget_id });
 
-        self.cache.add(widget_id);
+        self.context.cache.add(widget_id);
     }
 
     fn process_rebuild(&mut self, widget_id: WidgetId) {
+        // Grab the parent's depth
+        let parent_layer = {
+            let parent = self
+                .context
+                .tree
+                .get(&widget_id)
+                .expect("broken tree")
+                .parent;
+
+            match parent {
+                Some(parent_id) => self.get_node(&parent_id).layer,
+                None => 0,
+            }
+        };
+
         let node = self.widgets.get_mut(widget_id.id()).unwrap();
 
         // Queue the children for removal
         for child_id in &self
+            .context
             .tree
             .get(&widget_id)
             .expect("cannot destroy a widget that doesn't exist")
@@ -526,7 +514,16 @@ impl<'ui> WidgetManager<'ui> {
             self.modifications.push(Modify::Destroy(*child_id));
         }
 
-        match self.context.build(widget_id, &node.widget).take() {
+        *self.context.current_id.lock() = Some(ListenerId::Widget(widget_id));
+
+        let result = node
+            .widget
+            .try_get()
+            .map_or(BuildResult::Empty, |widget| widget.build(&self.context));
+
+        *self.context.current_id.lock() = None;
+
+        match result.take() {
             Ok(children) => {
                 for child in children {
                     if !child.is_valid() {
@@ -540,36 +537,39 @@ impl<'ui> WidgetManager<'ui> {
             Err(err) => panic!("build failed: {}", err),
         };
 
+        node.layer = parent_layer;
+
+        // If this widget has clipping set, increment its depth by one
+        if self.context.get_clipping(&widget_id).is_valid() {
+            node.layer += 1;
+        }
+
         // Store the node's layout so morphorm can access it
+        node.layout_type = self.context.get_layout_type(&widget_id);
         node.layout = self.context.get_layout(&widget_id);
     }
 
-    fn process_destroy<E>(&mut self, events: &mut E, widget_id: WidgetId)
-    where
-        E: Extend<WidgetEvent>,
-    {
+    fn process_destroy(&mut self, events: &mut Vec<WidgetEvent>, widget_id: WidgetId) {
         let widget = self
             .widgets
             .remove(widget_id.id())
             .expect("cannot remove a widget that does not exist");
 
         // Add the child widgets to the removal queue
-        if let Some(tree_node) = self.tree.remove(&widget_id) {
+        if let Some(tree_node) = self.context.tree.remove(&widget_id) {
             for child_id in tree_node.children {
                 self.modifications.push(Modify::Destroy(child_id));
             }
         }
 
-        self.cache.remove(&widget_id);
-
         self.context.remove(&widget_id);
 
-        self.changed.lock().remove(&ListenerID::Widget(widget_id));
+        self.changed.lock().remove(&ListenerId::Widget(widget_id));
 
-        events.extend(Some(WidgetEvent::Destroyed {
+        events.push(WidgetEvent::Destroyed {
             type_id: widget.widget.get_type_id(),
             widget_id,
-        }));
+        });
     }
 
     pub fn print_tree(&self) {
@@ -595,9 +595,7 @@ mod tests {
 
     use crate::{
         context::WidgetContext,
-        ui::VoidEvents,
-        unit::LayoutType,
-        widget::{BuildResult, Widget, WidgetBuilder, WidgetLayout, WidgetRef, WidgetType},
+        widget::{BuildResult, Widget, WidgetBuilder, WidgetRef, WidgetType},
     };
 
     use super::WidgetManager;
@@ -624,12 +622,6 @@ mod tests {
         }
     }
 
-    impl WidgetLayout for TestWidget {
-        fn layout_type(&self) -> LayoutType {
-            LayoutType::Column
-        }
-    }
-
     impl WidgetBuilder for TestWidget {
         fn build(&self, ctx: &WidgetContext) -> BuildResult {
             let computes = Arc::clone(&self.computes);
@@ -637,7 +629,7 @@ mod tests {
             let computed_value = ctx.computed(move |ctx| {
                 *computes.lock() += 1;
 
-                let test_global = ctx.get_global::<TestGlobal>();
+                let test_global = ctx.try_use_global::<TestGlobal>();
 
                 test_global.map_or_else(
                     || -1,
@@ -664,9 +656,16 @@ mod tests {
 
         assert_eq!(manager.additions, 0, "should not have added the widget");
 
-        manager.update(&mut VoidEvents);
+        let mut events = Vec::new();
 
-        let widget_id = &manager.tree.get_root().expect("failed to get root widget");
+        manager.update(&mut events);
+
+        events.clear();
+
+        let widget_id = &manager
+            .get_tree()
+            .get_root()
+            .expect("failed to get root widget");
 
         assert_eq!(manager.rebuilds, 1, "should have built the new widget");
 
@@ -693,7 +692,9 @@ mod tests {
             "widget `computes` should have been been 1"
         );
 
-        manager.update(&mut VoidEvents);
+        manager.update(&mut events);
+
+        events.clear();
 
         assert_eq!(manager.additions, 1, "should have 1 addition");
         assert_eq!(manager.removals, 0, "should have 0 removals");
@@ -717,18 +718,25 @@ mod tests {
     pub fn test_globals() {
         let mut manager = WidgetManager::new();
 
-        let test_global = manager.context.init_global::<TestGlobal>();
+        let test_global = manager.context.init_global(TestGlobal::default);
 
         manager.add(None, WidgetRef::new(TestWidget::default()));
 
-        manager.update(&mut VoidEvents);
+        let mut events = Vec::new();
+
+        manager.update(&mut events);
+
+        events.clear();
 
         assert_eq!(manager.additions, 1, "should have 1 addition");
         assert_eq!(manager.removals, 0, "should have 0 removals");
         assert_eq!(manager.rebuilds, 1, "should not have been rebuilt");
         assert_eq!(manager.changes, 0, "should not have changed");
 
-        let widget_id = &manager.tree.get_root().expect("failed to get root widget");
+        let widget_id = &manager
+            .get_tree()
+            .get_root()
+            .expect("failed to get root widget");
 
         // Compute function gets called twice, once for the default value and once to check if it needs
         // to be updated, after it detects a change in TestGlobal
@@ -759,7 +767,9 @@ mod tests {
             "widget `computes` should be 1"
         );
 
-        manager.update(&mut VoidEvents);
+        manager.update(&mut events);
+
+        events.clear();
 
         assert_eq!(manager.additions, 1, "should have 1 addition");
         assert_eq!(manager.removals, 0, "should have 0 removals");
