@@ -2,7 +2,7 @@ use generational_arena::Arena;
 
 use crate::{
     layout::Layout,
-    unit::{LayoutType, Padding, Position, Sizing},
+    unit::{LayoutType, Margin, Position, Sizing},
     widget::{WidgetId, WidgetRef},
     Ref,
 };
@@ -10,7 +10,8 @@ use crate::{
 /// Holds information about a widget in the UI tree.
 pub struct WidgetNode {
     pub widget: WidgetRef,
-    pub layout_type: LayoutType,
+    pub layer: u32,
+    pub layout_type: Ref<LayoutType>,
     pub layout: Ref<Layout>,
 }
 
@@ -21,7 +22,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
         Some(
             store
                 .get(self.id())
-                .map_or(LayoutType::default(), |node| node.layout_type)
+                .and_then(|node| node.layout_type.try_get())
+                .map_or(LayoutType::default(), |layout| *layout)
                 .into(),
         )
     }
@@ -42,7 +44,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Sizing::default(), |layout| layout.sizing)
-                .get_width(),
+                .get_width()
+                .into(),
         )
     }
 
@@ -52,7 +55,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Sizing::default(), |layout| layout.sizing)
-                .get_height(),
+                .get_height()
+                .into(),
         )
     }
 
@@ -62,7 +66,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Sizing::default(), |layout| layout.min_sizing)
-                .get_width(),
+                .get_width()
+                .into(),
         )
     }
 
@@ -72,7 +77,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Sizing::default(), |layout| layout.min_sizing)
-                .get_height(),
+                .get_height()
+                .into(),
         )
     }
 
@@ -82,7 +88,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Sizing::default(), |layout| layout.max_sizing)
-                .get_width(),
+                .get_width()
+                .into(),
         )
     }
 
@@ -92,7 +99,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Sizing::default(), |layout| layout.max_sizing)
-                .get_height(),
+                .get_height()
+                .into(),
         )
     }
 
@@ -102,7 +110,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Position::default(), |layout| layout.position)
-                .get_top(),
+                .get_top()
+                .into(),
         )
     }
 
@@ -112,7 +121,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Position::default(), |layout| layout.position)
-                .get_right(),
+                .get_right()
+                .into(),
         )
     }
 
@@ -122,7 +132,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Position::default(), |layout| layout.position)
-                .get_bottom(),
+                .get_bottom()
+                .into(),
         )
     }
 
@@ -132,7 +143,8 @@ impl<'a> morphorm::Node<'a> for WidgetId {
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
                 .map_or(Position::default(), |layout| layout.position)
-                .get_left(),
+                .get_left()
+                .into(),
         )
     }
 
@@ -173,8 +185,9 @@ impl<'a> morphorm::Node<'a> for WidgetId {
             store
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
-                .map_or(Padding::default(), |layout| layout.padding)
-                .get_top(),
+                .map_or(Margin::default(), |layout| layout.margin)
+                .get_top()
+                .into(),
         )
     }
 
@@ -183,8 +196,9 @@ impl<'a> morphorm::Node<'a> for WidgetId {
             store
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
-                .map_or(Padding::default(), |layout| layout.padding)
-                .get_right(),
+                .map_or(Margin::default(), |layout| layout.margin)
+                .get_right()
+                .into(),
         )
     }
 
@@ -193,8 +207,9 @@ impl<'a> morphorm::Node<'a> for WidgetId {
             store
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
-                .map_or(Padding::default(), |layout| layout.padding)
-                .get_bottom(),
+                .map_or(Margin::default(), |layout| layout.margin)
+                .get_bottom()
+                .into(),
         )
     }
 
@@ -203,37 +218,46 @@ impl<'a> morphorm::Node<'a> for WidgetId {
             store
                 .get(self.id())
                 .and_then(|node| node.layout.try_get())
-                .map_or(Padding::default(), |layout| layout.padding)
-                .get_left(),
+                .map_or(Margin::default(), |layout| layout.margin)
+                .get_left()
+                .into(),
         )
     }
 
     fn row_between(&self, store: &'_ Self::Data) -> Option<morphorm::Units> {
         store
             .get(self.id())
-            .map_or(LayoutType::default(), |node| node.layout_type)
-            .get_row_spacing()
+            .and_then(|node| node.layout_type.try_get())
+            .map_or(LayoutType::default(), |layout| *layout)
+            .get_column_spacing()
+            .map(|val| val.into())
     }
 
     fn col_between(&self, store: &'_ Self::Data) -> Option<morphorm::Units> {
         store
             .get(self.id())
-            .map_or(LayoutType::default(), |node| node.layout_type)
-            .get_column_spacing()
+            .and_then(|node| node.layout_type.try_get())
+            .map_or(LayoutType::default(), |layout| *layout)
+            .get_row_spacing()
+            .map(|val| val.into())
     }
 
     fn grid_rows(&self, store: &'_ Self::Data) -> Option<Vec<morphorm::Units>> {
         store
             .get(self.id())
-            .map_or(LayoutType::default(), |node| node.layout_type)
+            .and_then(|node| node.layout_type.try_get())
+            .map_or(LayoutType::default(), |layout| *layout)
             .get_rows()
+            .map(|val| val.into_iter().map(|val| val.into()).collect())
     }
 
     fn grid_cols(&self, store: &'_ Self::Data) -> Option<Vec<morphorm::Units>> {
         store
             .get(self.id())
-            .map_or(LayoutType::default(), |node| node.layout_type)
+            .and_then(|node| node.layout_type.try_get())
+            .map_or(LayoutType::default(), |layout| *layout)
             .get_columns()
+            .map(|val| val.into_iter().map(|val| val.into()).collect())
     }
 
     fn row_index(&self, _store: &'_ Self::Data) -> Option<usize> {
