@@ -1,16 +1,16 @@
 use morphorm::PositionType;
 
-pub use morphorm::Units;
+use super::Units;
 
 /// Indicates to the layout system how the children of a widget should be laid out.
 #[derive(Debug, Copy, Clone)]
 #[non_exhaustive]
 pub enum LayoutType {
     /// Widgets should be laid out side-by-side.
-    Row,
-    
+    Row { spacing: Units },
+
     /// Widgets should be laid out on top of one another.
-    Column,
+    Column { spacing: Units },
 
     /// Widgets should be laid out in a grid.
     Grid {
@@ -23,15 +23,17 @@ pub enum LayoutType {
 
 impl Default for LayoutType {
     fn default() -> Self {
-        Self::Column
+        Self::Column {
+            spacing: Units::Auto,
+        }
     }
 }
 
 impl From<LayoutType> for morphorm::LayoutType {
     fn from(ty: LayoutType) -> Self {
         match ty {
-            LayoutType::Row => Self::Row,
-            LayoutType::Column => Self::Column,
+            LayoutType::Row { .. } => Self::Row,
+            LayoutType::Column { .. } => Self::Column,
             LayoutType::Grid { .. } => Self::Grid,
         }
     }
@@ -41,7 +43,7 @@ impl LayoutType {
     #[must_use]
     pub fn get_rows(&self) -> Option<Vec<Units>> {
         match self {
-            LayoutType::Row | LayoutType::Column => None,
+            LayoutType::Row { .. } | LayoutType::Column { .. } => None,
             LayoutType::Grid { rows, .. } => {
                 let mut vec = Vec::with_capacity(*rows);
 
@@ -55,7 +57,8 @@ impl LayoutType {
     #[must_use]
     pub const fn get_row_spacing(&self) -> Option<Units> {
         match self {
-            LayoutType::Row | LayoutType::Column => None,
+            LayoutType::Row { spacing } => Some(*spacing),
+            LayoutType::Column { .. } => None,
             LayoutType::Grid { row_spacing, .. } => Some(*row_spacing),
         }
     }
@@ -63,7 +66,7 @@ impl LayoutType {
     #[must_use]
     pub fn get_columns(&self) -> Option<Vec<Units>> {
         match self {
-            LayoutType::Row | LayoutType::Column => None,
+            LayoutType::Row { .. } | LayoutType::Column { .. } => None,
             LayoutType::Grid { columns, .. } => {
                 let mut vec = Vec::with_capacity(*columns);
 
@@ -77,26 +80,27 @@ impl LayoutType {
     #[must_use]
     pub const fn get_column_spacing(&self) -> Option<Units> {
         match self {
-            LayoutType::Row | LayoutType::Column => None,
+            LayoutType::Row { .. } => None,
+            LayoutType::Column { spacing } => Some(*spacing),
             LayoutType::Grid { column_spacing, .. } => Some(*column_spacing),
         }
     }
 }
 
-/// Sets the padding around the elment.
+/// Sets the margin around the elment.
 #[derive(Debug, Copy, Clone)]
 #[non_exhaustive]
-pub enum Padding {
-    /// No padding.
+pub enum Margin {
+    /// No margin.
     Unset,
-    
-    /// Padding along the vertical and horizontal axis.
-    Axis {
-        vertical: Units,
-        horizontal: Units,
-    },
-    
-    /// Padding on every side.
+
+    /// Margin all around.
+    All(Units),
+
+    /// Margin along the vertical and horizontal axis.
+    Axis { vertical: Units, horizontal: Units },
+
+    /// Margin on every side.
     Set {
         top: Units,
         right: Units,
@@ -105,46 +109,84 @@ pub enum Padding {
     },
 }
 
-impl Default for Padding {
+impl Default for Margin {
     fn default() -> Self {
         Self::Unset
     }
 }
 
-impl Padding {
+impl Margin {
+    #[must_use]
+    pub fn center() -> Self {
+        Self::Axis {
+            vertical: Units::Stretch(1.0),
+            horizontal: Units::Stretch(1.0),
+        }
+    }
+
+    #[must_use]
+    pub fn h_center() -> Self {
+        Self::horizontal(Units::Stretch(1.0))
+    }
+
+    #[must_use]
+    pub fn v_center() -> Self {
+        Self::vertical(Units::Stretch(1.0))
+    }
+
+    #[must_use]
+    pub fn horizontal(units: Units) -> Self {
+        Self::Axis {
+            vertical: Units::default(),
+            horizontal: units,
+        }
+    }
+
+    #[must_use]
+    pub fn vertical(units: Units) -> Self {
+        Self::Axis {
+            vertical: units,
+            horizontal: Units::default(),
+        }
+    }
+
     #[must_use]
     pub const fn get_top(&self) -> Units {
         match self {
-            Padding::Unset => Units::Auto,
-            Padding::Axis { vertical, .. } => *vertical,
-            Padding::Set { top, .. } => *top,
+            Margin::Unset => Units::Auto,
+            Margin::All(units) => *units,
+            Margin::Axis { vertical, .. } => *vertical,
+            Margin::Set { top, .. } => *top,
         }
     }
 
     #[must_use]
     pub const fn get_right(&self) -> Units {
         match self {
-            Padding::Unset => Units::Auto,
-            Padding::Axis { horizontal, .. } => *horizontal,
-            Padding::Set { right, .. } => *right,
+            Margin::Unset => Units::Auto,
+            Margin::All(units) => *units,
+            Margin::Axis { horizontal, .. } => *horizontal,
+            Margin::Set { right, .. } => *right,
         }
     }
 
     #[must_use]
     pub const fn get_bottom(&self) -> Units {
         match self {
-            Padding::Unset => Units::Auto,
-            Padding::Axis { vertical, .. } => *vertical,
-            Padding::Set { bottom, .. } => *bottom,
+            Margin::Unset => Units::Auto,
+            Margin::All(units) => *units,
+            Margin::Axis { vertical, .. } => *vertical,
+            Margin::Set { bottom, .. } => *bottom,
         }
     }
 
     #[must_use]
     pub const fn get_left(&self) -> Units {
         match self {
-            Padding::Unset => Units::Auto,
-            Padding::Axis { horizontal, .. } => *horizontal,
-            Padding::Set { left, .. } => *left,
+            Margin::Unset => Units::Auto,
+            Margin::All(units) => *units,
+            Margin::Axis { horizontal, .. } => *horizontal,
+            Margin::Set { left, .. } => *left,
         }
     }
 }
@@ -155,7 +197,7 @@ impl Padding {
 pub enum Position {
     /// Position unchanged.
     Unset,
-    
+
     /// Position set absolutely in the window.
     Absolute {
         top: Units,
@@ -163,12 +205,9 @@ pub enum Position {
         bottom: Units,
         left: Units,
     },
-    
+
     /// Position set relative to its parent.
-    Relative {
-        top: Units,
-        left: Units,
-    },
+    Relative { top: Units, left: Units },
 }
 
 impl Default for Position {
@@ -221,7 +260,7 @@ impl Position {
 }
 
 /// The sizing of the element.
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq)]
 #[non_exhaustive]
 pub enum Sizing {
     /// Element size automatically set based onother factors.
@@ -230,8 +269,11 @@ pub enum Sizing {
     /// Element size attempts to fill its parent container.
     Fill,
 
-    /// Element has a set, specific size.
-    Set { width: Units, height: Units },
+    /// Element has the same sizing for width and height.
+    All(Units),
+
+    /// Element has a sizings for each of its axis.
+    Axis { width: Units, height: Units },
 }
 
 impl Default for Sizing {
@@ -246,7 +288,8 @@ impl Sizing {
         match self {
             Sizing::Auto => Units::Auto,
             Sizing::Fill => Units::Stretch(1.0),
-            Sizing::Set { width, .. } => *width,
+            Sizing::All(units) => *units,
+            Sizing::Axis { width, .. } => *width,
         }
     }
 
@@ -255,7 +298,8 @@ impl Sizing {
         match self {
             Sizing::Auto => Units::Auto,
             Sizing::Fill => Units::Stretch(1.0),
-            Sizing::Set { height, .. } => *height,
+            Sizing::All(units) => *units,
+            Sizing::Axis { height, .. } => *height,
         }
     }
 }
