@@ -7,10 +7,7 @@ use std::{
 use downcast_rs::{impl_downcast, Downcast};
 use generational_arena::Index as GenerationalIndex;
 
-use crate::{
-    context::WidgetContext,
-    unit::{Key, LayoutType},
-};
+use crate::{context::WidgetContext, unit::Key};
 
 /// Encapsulates the result of a widget `build()` method.
 #[non_exhaustive]
@@ -20,14 +17,14 @@ pub enum BuildResult {
 
     /// The widget contains a single child.
     One(WidgetRef),
-    
+
     /// The widget contains a many children.
     Many(Vec<WidgetRef>),
 
     /// The widget has failed to build properly, and should construction should halt.
-    /// 
+    ///
     /// # Panics
-    /// 
+    ///
     /// Currently this results in a `panic!()`, however that may change in the future.
     Error(Box<dyn std::error::Error>),
 }
@@ -82,21 +79,15 @@ impl From<&Vec<WidgetRef>> for BuildResult {
 pub trait WidgetType {
     /// Return the `TypeId::of()` of the widget.
     fn get_type_id(&self) -> TypeId;
-    
+
     /// Return the name of the widget as a string. Generally this is the name of the struct.
     fn get_type_name(&self) -> &'static str;
 }
 
-/// Implements functions necessary for the layout system. These generally won't change `build()`-to-`build()`.
-pub trait WidgetLayout {
-    /// Dictate how the children widgets should be laid out.
-    fn layout_type(&self) -> LayoutType;
-}
-
 /// Implements the widget's `build()` method.
 pub trait WidgetBuilder: Downcast {
     /// Called whenever this widget is rebuilt.
-    /// 
+    ///
     /// This method may be called when any parent is rebuilt, when its internal state changes, when
     /// global state changes, when a computed function changes, or just because it feels like it. Hence,
     /// it should not be relied on for any reason other than to return child widgets.
@@ -104,12 +95,12 @@ pub trait WidgetBuilder: Downcast {
 }
 
 /// The combined Widget implementation, required to be used within the `WidgetBuilder`.
-pub trait Widget: WidgetType + WidgetLayout + WidgetBuilder {}
+pub trait Widget: WidgetType + WidgetBuilder {}
 
 impl_downcast!(Widget);
 
 /// Holds a reference to a widget, or not.
-/// 
+///
 /// This is generally used when a widget can accept children as a parameter. It can either be `Owned`,
 /// `Borrowed`, `None`, or `Keyed`. A `Keyed` widget is one that may retain its state across parental rebuilds.
 pub enum WidgetRef {
@@ -118,7 +109,7 @@ pub enum WidgetRef {
 
     /// An owned widget.
     Owned(Rc<dyn Widget>),
-    
+
     /// A borrowed widget.
     Borrowed(Weak<dyn Widget>),
 
@@ -262,7 +253,7 @@ impl Into<Vec<Self>> for WidgetRef {
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
-pub struct WidgetId(GenerationalIndex, usize);
+pub struct WidgetId(GenerationalIndex);
 
 impl std::fmt::Display for WidgetId {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -272,29 +263,24 @@ impl std::fmt::Display for WidgetId {
 
 impl WidgetId {
     #[must_use]
-    pub const fn from(index: GenerationalIndex, depth: usize) -> Self {
-        Self(index, depth)
+    pub const fn from(index: GenerationalIndex) -> Self {
+        Self(index)
     }
 
     #[must_use]
     pub const fn id(&self) -> GenerationalIndex {
         self.0
     }
-
-    #[must_use]
-    pub const fn depth(&self) -> usize {
-        self.1
-    }
 }
 
 impl Default for WidgetId {
     fn default() -> Self {
-        Self(GenerationalIndex::from_raw_parts(0, 0), 0)
+        Self(GenerationalIndex::from_raw_parts(0, 0))
     }
 }
 
 impl From<usize> for WidgetId {
     fn from(val: usize) -> Self {
-        Self(GenerationalIndex::from_raw_parts(val, 0), 0)
+        Self(GenerationalIndex::from_raw_parts(val, 0))
     }
 }
